/*
 * IRC - Internet Relay Chat, include/msg.h
 * Copyright (C) 1990 Jarkko Oikarinen and
 *                    University of Oulu, Computing Center
 *
 * See file doc/authors for additional names of
 * the programmers.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef MSG_H
#define MSG_H

/*=============================================================================
 * General defines
 */

#define MAXPARA	   15

/*-----------------------------------------------------------------------------
 * Macro's
 */

/*
 * Tokenization:
 * Each command must have a TOK_COMMAND and MSG_COMMAND definition.
 * If you don't want one or the other, make them the same.
 * Also each command has a "msgclass" used for debugging purposes.
 */

/* *INDENT-OFF* */

#define MSG_PRIVATE		"PRIVMSG"	/* PRIV */
#define TOK_PRIVATE		"P"
#define CLASS_PRIVATE		LEVEL_PROPAGATE
#define HELP_PRIVATE		"Sends a message."

#define MSG_WHO			"WHO"		/* WHO	-> WHOC */
#define TOK_WHO			"H"
#define CLASS_WHO		LEVEL_QUERY
#define HELP_WHO		"Gives a list of who is online."

#define MSG_WHOIS		"WHOIS"		/* WHOI */
#define TOK_WHOIS		"W"
#define CLASS_WHOIS		LEVEL_QUERY
#define HELP_WHOIS		"Gives detailed information about someone."

#define MSG_WHOWAS		"WHOWAS"	/* WHOW */
#define TOK_WHOWAS		"X"
#define CLASS_WHOWAS		LEVEL_QUERY
#define HELP_WHOWAS		"Gives information about someone who was online."

#define MSG_USER		"USER"		/* USER */
#define TOK_USER		"USER"
#define CLASS_USER		LEVEL_CLIENT
#define HELP_USER		"Used during connecting to pass user information to the server."

#define MSG_NICK		"NICK"		/* NICK */
#define TOK_NICK		"N"
#define CLASS_NICK		LEVEL_CLIENT
#define HELP_NICK		"Changes a nick."

#define MSG_SERVER		"SERVER"	/* SERV */
#define TOK_SERVER		"S"
#define CLASS_SERVER		LEVEL_MAP
#define HELP_SERVER		"Command used for server-server links."

#define MSG_LIST		"LIST"		/* LIST */
#define TOK_LIST		"LS"
#define CLASS_LIST		LEVEL_QUERY
#define HELP_LIST		"Gives a list of channels."

#define MSG_TOPIC		"TOPIC"		/* TOPI */
#define TOK_TOPIC		"T"
#define CLASS_TOPIC		LEVEL_PROPAGATE
#define HELP_TOPIC		"Changes the topic of a channel."

#define MSG_INVITE		"INVITE"	/* INVI */
#define TOK_INVITE		"I"
#define CLASS_INVITE		LEVEL_MODE
#define HELP_INVITE		"Invites someone to a channel."

#define MSG_CHKCHAN             "CHECK"		/* CHKC */
#define TOK_CHKCHAN             "CC"
#define CLASS_CHKCHAN           LEVEL_QUERY
#define HELP_CHKCHAN		"Gives extended information on a user/channel/server."

#define MSG_VERSION		"VERSION"	/* VERS */
#define TOK_VERSION		"V"
#define CLASS_VERSION		LEVEL_QUERY
#define HELP_VERSION		"Passes the version of a server."

#define MSG_GVERSION		"GVERSION"	/* GVER */
#define TOK_GVERSION		"GV"
#define CLASS_GVERSION		LEVEL_QUERY
#define HELP_GVERSION		"Passes the version of all the connected servers."

#define MSG_QUIT		"QUIT"		/* QUIT */
#define TOK_QUIT		"Q"
#define CLASS_QUIT		LEVEL_CLIENT
#define HELP_QUIT		"Quits a chatsession."

#define MSG_SQUIT		"SQUIT"		/* SQUI */
#define TOK_SQUIT		"SQ"
#define CLASS_SQUIT		LEVEL_MAP
#define HELP_SQUIT		"Disconnects a serverlink."

#define MSG_KILL		"KILL"		/* KILL */
#define TOK_KILL		"D"
#define CLASS_KILL		LEVEL_CLIENT
#define HELP_KILL		"Sends a KILL message to a user, which will cause the user to be disconnected from IRC."

#define MSG_INFO		"INFO"		/* INFO */
#define TOK_INFO		"F"
#define CLASS_INFO		LEVEL_QUERY
#define HELP_INFO		"Returns info about the ircd."

#define MSG_LINKS		"LINKS"		/* LINK */
#define TOK_LINKS		"LI"
#define CLASS_LINKS		LEVEL_QUERY
#define HELP_LINKS		"Returns the services and the servers that are currently linked to the network."

#define MSG_STATS		"STATS"		/* STAT */
#define TOK_STATS		"R"
#define CLASS_STATS		LEVEL_QUERY
#define HELP_STATS		"Shows info on server statistics and settings."

#define MSG_HELP		"HELP"		/* HELP */
#define TOK_HELP		"HE"
#define CLASS_HELP		LEVEL_QUERY
#define HELP_HELP		"These messages."

#define MSG_ERROR		"ERROR"		/* ERRO */
#define TOK_ERROR		"Y"
#define CLASS_ERROR		LEVEL_PROPAGATE
#define HELP_ERROR		"Terminates a client forcefully."

#define MSG_AWAY		"AWAY"		/* AWAY */
#define TOK_AWAY		"A"
#define CLASS_AWAY		LEVEL_PROPAGATE
#define HELP_AWAY		"Tells the server you are (no longer) away."

#define MSG_CONNECT		"CONNECT"	/* CONN */
#define TOK_CONNECT		"CO"
#define CLASS_CONNECT		LEVEL_PROPAGATE
#define HELP_CONNECT		"Connects a server to another."

#define MSG_UPING		"UPING"		/* UPIN */
#define TOK_UPING		"UP"
#define CLASS_UPING		LEVEL_PROPAGATE
#define HELP_UPING		"Does an UDP based ping to a remote server"

#define MSG_MAP			"MAP"		/* MAP	*/
#define TOK_MAP			"MA"
#define CLASS_MAP		LEVEL_QUERY
#define HELP_MAP		"Returns a map of services and servers that are currently linked to the network."

#define MSG_PING		"PING"		/* PING */
#define TOK_PING		"G"
#define CLASS_PING		LEVEL_PROPAGATE
#define HELP_PING		"Command sent by the server to check if the connection is still active."

#define MSG_PONG		"PONG"		/* PONG */
#define TOK_PONG		"Z"
#define CLASS_PONG		LEVEL_CLIENT
#define HELP_PONG		"Command sent by the client/server on the other side to confirm the connection is still active."

#define MSG_OPER		"OPER"		/* OPER */
#define TOK_OPER		"OP"
#define CLASS_OPER		LEVEL_PROPAGATE
#define HELP_OPER		"Enables the usermode +o/+O of an ircop."

#define MSG_LOGIN		"LOGIN"		/* LOGI */
#define TOK_LOGIN		"LG"
#define CLASS_LOGIN		LEVEL_PROPAGATE
#define HELP_LOGIN		"Allows a non-ircop to get extra flags (based on X-lines)."

#define MSG_PASS		"PASS"		/* PASS */
#define TOK_PASS		"PA"
#define CLASS_PASS		LEVEL_CLIENT
#define HELP_PASS		"Used to enter a password during connect."

#define MSG_WALLOPS		"WALLOPS"	/* WALL */
#define TOK_WALLOPS		"WA"
#define CLASS_WALLOPS		LEVEL_PROPAGATE
#define HELP_WALLOPS		"Sends a message to all users that are +w."

#define MSG_GLOBOPS		"GLOBOPS"	/* WALL */
#define TOK_GLOBOPS		"GO"
#define CLASS_GLOBOPS		LEVEL_PROPAGATE
#define HELP_GLOBOPS		"Sends a networkwide global message to ircops."

#define MSG_CHATOPS		"CHATOPS"	/* WALL */
#define TOK_CHATOPS		"CH"
#define CLASS_CHATOPS		LEVEL_PROPAGATE
#define HELP_CHATOPS		"Sends a networkwide chat message to ircops."

#define MSG_LOCOPS		"LOCOPS"	/* WALL */
#define TOK_LOCOPS		"LO"
#define CLASS_LOCOPS		LEVEL_PROPAGATE
#define HELP_LOCOPS		"Sends a serverwide local message to ircops."

#define MSG_DESYNCH     	"DESYNCH"       /* DESY */
#define TOK_DESYNCH     	"DS"
#define CLASS_DESYNCH   	LEVEL_PROPAGATE
#define HELP_DESYNCH		"Writes to all +e users; for sending wall type debugging/anti-hack info."

#define MSG_TIME			"TIME"		/* TIME */
#define TOK_TIME			"TI"
#define CLASS_TIME		LEVEL_QUERY
#define HELP_TIME			"Returns the current servertime."

#define MSG_SETTIME		"SETTIME"	/* SETT */
#define TOK_SETTIME		"SE"
#define CLASS_SETTIME		LEVEL_PROPAGATE
#define HELP_SETTIME		"Synchronises the servertime over the net."

#define MSG_RPING			"RPING"	/* RPIN */
#define TOK_RPING			"RI"
#define CLASS_RPING		LEVEL_PROPAGATE
#define HELP_RPING		"Sends a word to another remote server which the remote server will bounce back."

#define MSG_GRPING		"GRPING"	/* GRPI */
#define TOK_GRPING		"GR"
#define CLASS_GRPING		LEVEL_PROPAGATE
#define HELP_GRPING		"Sends a word to all remote servers which they will bounce back."

#define MSG_RPONG			"RPONG"	/* RPON */
#define TOK_RPONG			"RO"
#define CLASS_RPONG		LEVEL_PROPAGATE
#define HELP_RPONG		"Server reply to a RPING."

#define MSG_NAMES			"NAMES"	/* NAME */
#define TOK_NAMES			"E"
#define CLASS_NAMES		LEVEL_QUERY
#define HELP_NAMES		"Lists the nicks in a channel."

#define MSG_ADMIN			"ADMIN"	/* ADMI */
#define TOK_ADMIN			"AD"
#define CLASS_ADMIN		LEVEL_QUERY
#define HELP_ADMIN		"Shows who the admin is for a server."

#define MSG_TRACE			"TRACE"	/* TRAC */
#define TOK_TRACE			"TR"
#define CLASS_TRACE		LEVEL_PROPAGATE
#define HELP_TRACE		"Traces a path from the current server to the specified server or user."

#define MSG_NOTICE		"NOTICE"	/* NOTI */
#define TOK_NOTICE		"O"
#define CLASS_NOTICE		LEVEL_PROPAGATE
#define HELP_NOTICE		"Sends a notice."

#define MSG_WALLCHOPS		"WALLCHOPS"	/* WC */
#define TOK_WALLCHOPS		"WC"
#define CLASS_WALLCHOPS		LEVEL_PROPAGATE
#define HELP_WALLCHOPS		"Sends a message to the channel ops."

#define MSG_CPRIVMSG		"CPRIVMSG"	/* CPRI */
#define TOK_CPRIVMSG		"CP"
#define CLASS_CPRIVMSG		LEVEL_CLIENT
#define HELP_CPRIVMSG		"Send a message to a client on a specified channel while bypassing anti-spam checking."

#define MSG_CNOTICE		"CNOTICE"	/* CNOT */
#define TOK_CNOTICE		"CN"
#define CLASS_CNOTICE		LEVEL_CLIENT
#define HELP_CNOTICE		"Send a notice to a client on a specified channel while bypassing anti-spam checking."

#define MSG_JOIN			"JOIN"	/* JOIN */
#define TOK_JOIN			"J"
#define CLASS_JOIN		LEVEL_CHANNEL
#define HELP_JOIN			"Join a channel."

#define MSG_PART			"PART"	/* PART */
#define TOK_PART			"L"
#define CLASS_PART		LEVEL_CHANNEL
#define HELP_PART			"Part a channel."

#define MSG_LUSERS		"LUSERS"	/* LUSE */
#define TOK_LUSERS		"LU"
#define CLASS_LUSERS		LEVEL_QUERY
#define HELP_LUSERS		"Lists usercount info."

#define MSG_MOTD			"MOTD"	/* MOTD */
#define TOK_MOTD			"MO"
#define CLASS_MOTD		LEVEL_QUERY
#define HELP_MOTD			"Shows the message of the day (MOTD)."

#define MSG_MODE			"MODE"	/* MODE */
#define TOK_MODE			"M"
#define CLASS_MODE		LEVEL_MODE
#define HELP_MODE			"Sets a mode."

#define MSG_OPME			"OPME"	/* OPME */
#define TOK_OPME			"OM"
#define CLASS_OPME		LEVEL_MODE
#define HELP_OPME			"Allows an oper with security override on to op himself."

#define MSG_KICK			"KICK"		/* KICK */
#define TOK_KICK			"K"
#define CLASS_KICK		LEVEL_CHANNEL
#define HELP_KICK			"Removes someone from a channel."

#define MSG_USERHOST		"USERHOST"	/* USER -> USRH */
#define TOK_USERHOST		"USERHOST"
#define CLASS_USERHOST		LEVEL_QUERY
#define HELP_USERHOST		"Shows someone's user and host."

#define MSG_USERIP		"USERIP"	/* USER -> USIP */
#define TOK_USERIP		"USERIP"
#define CLASS_USERIP		LEVEL_QUERY
#define HELP_USERIP		"Shows someone's user and ip."

#define MSG_ISON		"ISON"		/* ISON */
#define TOK_ISON		"ISON"
#define CLASS_ISON		LEVEL_QUERY
#define HELP_ISON		"Shows if someone is on."

#define MSG_REHASH		"REHASH"	/* REHA */
#define TOK_REHASH		"REHASH"
#define CLASS_REHASH		LEVEL_MAP
#define HELP_REHASH		"Rehashes the ircd."

#define MSG_RESTART		"RESTART"	/* REST */
#define TOK_RESTART		"RESTART"
#define CLASS_RESTART		LEVEL_MAP
#define HELP_RESTART		"Restarts the ircd."

#define MSG_CLOSE		"CLOSE"		/* CLOS */
#define TOK_CLOSE		"CLOSE"
#define CLASS_CLOSE		LEVEL_CLIENT
#define HELP_CLOSE		"Closes all unknown connection."

#define MSG_DIE			"DIE"		/* DIE	*/
#define TOK_DIE			"DIE"
#define CLASS_DIE		LEVEL_MAP
#define HELP_DIE		"Kills the server."

#define MSG_HASH		"HASH"		/* HASH */
#define TOK_HASH		"HASH"
#define CLASS_HASH		LEVEL_QUERY
#define HELP_HASH		"Shows the hash table statistics."

#define MSG_DNS			"DNS"		/* DNS	-> DNSS */
#define TOK_DNS			"DNS"
#define CLASS_DNS		LEVEL_QUERY
#define HELP_DNS		"Displays dns cache info."

#define MSG_SILENCE		"SILENCE"	/* SILE */
#define TOK_SILENCE		"U"
#define CLASS_SILENCE		LEVEL_PROPAGATE
#define HELP_SILENCE		"Handles server side silence masks."

#define MSG_GLINE		"GLINE"		/* GLIN */
#define TOK_GLINE		"GL"
#define CLASS_GLINE		LEVEL_CLIENT
#define HELP_GLINE		"Adds a network wide ban."

#define MSG_RGLOBAL		"RGLOBAL"	/* RGLO */
#define TOK_RGLOBAL		"RG"
#define CLASS_RGLOBAL		LEVEL_CLIENT
#define HELP_RGLOBAL		"Resets the global max lusers."

#define MSG_RLOCAL		"RLOCAL"	/* RLOC */
#define TOK_RLOCAL		"RL"
#define CLASS_RLOCAL		LEVEL_CLIENT
#define HELP_RLOCAL		"Resets the local max lusers."

#define MSG_BURST		"BURST"		/* BURS */
#define TOK_BURST		"B"
#define CLASS_BURST		LEVEL_CHANNEL
#define HELP_BURST		"Sends channel information during the linking of servers."

#ifdef TOPICSYNC
#define MSG_TBURST		"TBURST"	/* TBUR */
#define TOK_TBURST		"TB"
#define CLASS_TBURST		LEVEL_CHANNEL
#define HELP_TBURST		"Sends channel topic information during the linking of servers."
#endif

#define MSG_CREATE		"CREATE"	/* CREA */
#define TOK_CREATE		"C"
#define CLASS_CREATE		LEVEL_CHANNEL
#define HELP_CREATE		"Creates a new channel."

#define MSG_DESTRUCT		"DESTRUCT"	/* DEST */
#define TOK_DESTRUCT		"DE"
#define CLASS_DESTRUCT		LEVEL_CHANNEL
#define HELP_DESTRUCT		"Reserved for future usage."

#define MSG_END_OF_BURST	"END_OF_BURST"	/* END_ */
#define TOK_END_OF_BURST	"EB"
#define CLASS_END_OF_BURST	LEVEL_MAP
#define HELP_END_OF_BURST	"A message sent one one server to another to indicate that it has sent its bursts."

#define MSG_END_OF_BURST_ACK	"EOB_ACK"	/* EOB_ */
#define TOK_END_OF_BURST_ACK	"EA"
#define CLASS_END_OF_BURST_ACK	LEVEL_MAP
#define HELP_END_OF_BURST_ACK	"A message sent one one server to another to indicate that it has received the eob."

#define MSG_CRYPT		"CRYPT"		/* CRYP */
#define TOK_CRYPT		"CR"
#define CLASS_CRYPT		LEVEL_QUERY
#define HELP_CRYPT		"Encrypts a text string."

#define MSG_CS			"CS"		/* CS__ */
#define TOK_CS			"CS"
#define CLASS_CS		LEVEL_MAP
#define HELP_CS			"Relays a message to chanserv."

#define MSG_HS			"HS"		/* HS__ */
#define TOK_HS			"HS"
#define CLASS_HS		LEVEL_MAP
#define HELP_HS			"Relays a message to helpserv."

#define MSG_MS			"MS"		/* MS__ */
#define TOK_MS			"MS"
#define CLASS_MS		LEVEL_MAP
#define HELP_MS			"Relays a message to memoserv."

#define MSG_NS			"NS"		/* NS__ */
#define TOK_NS			"NS"
#define CLASS_NS		LEVEL_MAP
#define HELP_NS			"Relays a message to nickserv."

#define MSG_OS			"OS"		/* OS__ */
#define TOK_OS			"OS"
#define CLASS_OS		LEVEL_MAP
#define HELP_OS			"Relays a message to operserv."

#define MSG_SS			"SS"		/* SS__ */
#define TOK_SS			"SS"
#define CLASS_SS		LEVEL_MAP
#define HELP_SS			"Relays a message to statserv."

#define MSG_X2			"X2"		/* X2__ */
#define TOK_X2			"X2"
#define CLASS_X2		LEVEL_MAP
#define HELP_X2			"Relays a message to X2."

#define MSG_ADDLINE		"ADDLINE"	/* ADDL */
#define TOK_ADDLINE		"AL"
#define CLASS_ADDLINE		LEVEL_MAP
#define HELP_ADDLINE		"Not implemented."

#define MSG_REMLINE		"REMLINE"	/* REML */
#define TOK_REMLINE		"RL"
#define CLASS_REMLINE		LEVEL_MAP
#define HELP_REMLINE		"Not implemented."

#define MSG_VIEWLINE		"VIEWLINE"	/* VIEW */
#define TOK_VIEWLINE		"VL"
#define CLASS_VIEWLINE		LEVEL_MAP
#define HELP_VIEWLINE		"Not implemented."

#define MSG_ADDHLINE		"ADDHLINE"	/* ADDH */
#define TOK_ADDHLINE		"AH"
#define CLASS_ADDHLINE		LEVEL_MAP
#define HELP_ADDHLINE		"Not implemented."

#define MSG_ADDQLINE		"ADDQLINE"	/* ADDQ */
#define TOK_ADDQLINE		"AQ"
#define CLASS_ADDQLINE		LEVEL_MAP
#define HELP_ADDQLINE		"Not implemented."

#define MSG_ADDULINE		"ADDULINE"	/* ADDU */
#define TOK_ADDULINE		"AU"
#define CLASS_ADDULINE		LEVEL_MAP
#define HELP_ADDULINE		"Not implemented."

#define MSG_DISABLEO		"DISABLEO"	/* DISO */
#define TOK_DISABLEO		"DO"
#define CLASS_DISABLEO		LEVEL_MAP
#define HELP_DISABLEO		"Not implemented."

#define MSG_ENABLEO		"ENABLEO"	/* ENAO */
#define TOK_ENABLEO		"EO"
#define CLASS_ENABLEO		LEVEL_MAP
#define HELP_ENABLEO		"Not implemented."

#define MSG_REMHLINE		"REMHLINE"	/* REMH */
#define TOK_REMHLINE		"RH"
#define CLASS_REMHLINE		LEVEL_MAP
#define HELP_REMHLINE		"Not implemented."

#define MSG_REMQLINE		"REMQLINE"	/* REMQ */
#define TOK_REMQLINE		"RQ"
#define CLASS_REMQLINE		LEVEL_MAP
#define HELP_REMQLINE		"Not implemented."

#define MSG_REMULINE		"REMULINE"	/* REMU */
#define TOK_REMULINE		"RU"
#define CLASS_REMULINE		LEVEL_MAP
#define HELP_REMULINE		"Not implemented."

#define MSG_GREHASH		"GREHASH"	/* GREH */
#define TOK_GREHASH		"GH"
#define CLASS_GREHASH		LEVEL_MAP
#define HELP_GREHASH		"Not implemented."

#define MSG_RULES		"RULES"		/* RULE */
#define TOK_RULES		"RU"
#define CLASS_RULES		LEVEL_QUERY
#define HELP_RULES		"Displays the network rules."

#define MSG_SJOIN		"SJOIN"		/* SJOI */
#define TOK_SJOIN		"SJOIN"
#define CLASS_SJOIN		LEVEL_MAP
#define HELP_SJOIN		"Not implemented."

#define MSG_SMODE		"SMODE"		/* SMOD */
#define TOK_SMODE		"SM"
#define CLASS_SMODE		LEVEL_MAP
#define HELP_SMODE		"Allows a service to set a mode on a client."

#define MSG_SPART		"SPART"		/* SPAR */
#define TOK_SPART		"SP"
#define CLASS_SPART		LEVEL_MAP
#define HELP_SPART		"Not implemented."

/* *INDENT-ON* */

/*=============================================================================
 * Constants
 */
#define   MFLG_SLOW              0x01	/* Command can be executed roughly    *
					 * once per 2 seconds.                */
#define   MFLG_UNREG             0x02	/* Command available to unregistered  *
					 * clients.                           */

/*=============================================================================
 * Structures
 */

struct Message {
  unsigned int msgclass;
  char *cmd;
  char *tok;
  int (*func) (aClient *cptr, aClient *sptr, int parc, char *parv[]);
  /* cptr = Connected client ptr
     sptr = Source client ptr
     parc = parameter count
     parv = parameter variable array */
  unsigned int count;
  unsigned int parameters;
  unsigned char flags;		/* bit 0 set means that this command is allowed
				   to be used only on the average of once per 2
				   seconds -SRB */
  unsigned int bytes;
  char *helpline;
};

struct MessageTree {
  char *final;
  struct Message *msg;
  struct MessageTree *pointers[26];
};

/*=============================================================================
 * Proto types
 */

extern struct Message msgtab[];

#endif /* MSG_H */
