/*
 * IRC - Internet Relay Chat, include/s_bsd.h
 *
 * See file doc/authors for names of the programmers.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef S_BSD_H
#define S_BSD_H

#include <netdb.h>
#include "s_conf.h"

/*=============================================================================
 * Macro's
 */

/*
 * 2 sets of flags to not cross the 32bit boundary
 * [Term] December 22th 2001
 */

/* usermode part           12345678 */
#define FLAGS_DEBUG      0x00000001	/* send global debug/anti-hack info */
#define FLAGS_DEAF       0x00000002	/* Makes user deaf */
#define FLAGS_CHSERV     0x00000004	/* Disallow KICK or MODE -o on the user; don't display channels in /whois */
#define FLAGS_OPER       0x00000008	/* Operator */
#define FLAGS_LOCOP      0x00000010	/* Local operator -- SRB */
#define FLAGS_INVISIBLE  0x00000020	/* makes user invisible */
#define FLAGS_WALLOP     0x00000040	/* send wallops to them */
#define FLAGS_SERVNOTICE 0x00000080	/* server notices such as kill */
#define FLAGS_NOPRIVATES 0x00000100	/* RaptorNet's flag to disable incoming privates*/
#define FLAGS_NOCTCPDCC  0x00000200	/* RaptorNet's flag to disable DCC and CTCP */
#define FLAGS_REGGED     0x00000400	/* Registered nick */
#define FLAGS_HIDEHOST   0x00000800	/* Flag to enable host hiding */
#define FLAGS_GODMODE    0x00001000	/* AfterNET's "God" Mode for IRCops */
#define FLAGS_BOT        0x00002000	/* Bot flag to indicate BitchX and eggdrop */
#define FLAGS_SBOT       0x00004000	/* ServicesBot */
#define FLAGS_RCENTRAL   0x00008000	/* Raptor-Central member */
#define FLAGS_ADMIN      0x00010000	/* Server Admin */
#define FLAGS_COADMIN    0x00020000	/* Server CoAdmin */
#define FLAGS_BACKUPOP   0x00040000	/* Backup ircop */
#define FLAGS_CODER      0x00080000	/* Coder-Com member */
#define FLAGS_HELPER     0x00100000	/* HelpDesk member */
#define FLAGS_USERCOM    0x00200000	/* User-Com member */	
#define FLAGS_WEBCOM     0x00400000	/* Web-Com member */
#define FLAGS_BSERVICE   0x00800000	/* BService member */
#define FLAGS_CSERVICE   0x01000000	/* CService member */
#define FLAGS_ESERVICE   0x02000000	/* Egg-Com member */
#define FLAGS_WHOIS      0x04000000	/* wants to see /whois notice */

/* other part              12345678 */
#define FLAGS_PINGSENT   0x00000001	/* Unreplied ping sent */
#define FLAGS_DEADSOCKET 0x00000002	/* Local socket is dead--Exiting soon */
#define FLAGS_KILLED     0x00000004	/* Prevents "QUIT" from being sent for this */
#define FLAGS_BLOCKED    0x00000008	/* socket is in a blocked condition */
#define FLAGS_UNIX       0x00000010	/* socket is in the unix domain, not inet */
#define FLAGS_CLOSING    0x00000020	/* set when closing to suppress errors */
#define FLAGS_LISTEN     0x00000040	/* used to mark clients which we listen() on */
#define FLAGS_CHKACCESS  0x00000080	/* ok to check clients access if set */
#define FLAGS_DOINGDNS   0x00000100	/* client is waiting for a DNS response */
#define FLAGS_AUTH       0x00000200	/* client is waiting on rfc931 response */
#define FLAGS_WRAUTH     0x00000400	/* set if we havent writen to ident server */
#define FLAGS_LOCAL      0x00000800	/* set for local clients */
#define FLAGS_GOTID      0x00001000	/* successful ident lookup achieved */
#define FLAGS_DOID       0x00002000	/* I-lines say must use ident return */
#define FLAGS_NONL       0x00004000	/* No \n in buffer */
#define FLAGS_TS8        0x00008000	/* Why do you want to know? */
#define FLAGS_PING       0x00010000	/* Socket needs to send udp pings */
#define FLAGS_ASKEDPING  0x00020000	/* Client asked for udp ping */
#define FLAGS_MAP        0x00040000	/* Show server on the map */
#define FLAGS_JUNCTION   0x00080000	/* Junction causing the net.burst */
#define FLAGS_BURST      0x00100000	/* Server is receiving a net.burst */
#define FLAGS_BURST_ACK  0x00200000	/* Server is waiting for eob ack */
#define FLAGS_IPCHECK    0x00400000	/* Added or updated IPregistry data */
#define FLAGS_SOCKS      0x00800000	/* check socks */
#define FLAGS_WRSOCKS    0x01000000	/* write to socks */
#define FLAGS_VLINED     0x02000000	/* user has a vline */


#define SEND_UMODES \
    (FLAGS_DEBUG | FLAGS_DEAF | FLAGS_CHSERV | FLAGS_OPER | /* FLAGS_LOCOP | */ \
	 FLAGS_INVISIBLE | FLAGS_WALLOP | /* FLAGS_SERVNOTICE | */ FLAGS_NOPRIVATES | \
	 FLAGS_NOCTCPDCC | FLAGS_REGGED | FLAGS_HIDEHOST | FLAGS_GODMODE | \
	 FLAGS_BOT | FLAGS_SBOT | FLAGS_RCENTRAL | FLAGS_ADMIN | FLAGS_COADMIN | \
	 FLAGS_BACKUPOP | FLAGS_CODER | FLAGS_HELPER | FLAGS_USERCOM | FLAGS_WEBCOM | \
	 FLAGS_BSERVICE | FLAGS_CSERVICE | FLAGS_ESERVICE | FLAGS_WHOIS)
#define ALL_UMODES (SEND_UMODES | FLAGS_SERVNOTICE | FLAGS_LOCOP)
#define FLAGS_ID (FLAGS_DOID | FLAGS_GOTID)


/*
 * flags macros.
 */
#define SendDebug(x)           ((x)->flags & FLAGS_DEBUG)
#define IsDeaf(x)              ((x)->flags & FLAGS_DEAF)
#define IsChannelService(x)    ((x)->flags & FLAGS_CHSERV)
#define IsNoKick(x)            ((x)->flags & FLAGS_CHSERV)
#define IsOper(x)              ((x)->flags & FLAGS_OPER)
#define IsLocOp(x)             ((x)->flags & FLAGS_LOCOP)
#define IsAnOper(x)            ((x)->flags & (FLAGS_OPER|FLAGS_LOCOP))
#define IsPrivileged(x)        (IsAnOper(x) || IsServer(x))
#define IsInvisible(x)         ((x)->flags & FLAGS_INVISIBLE)
#define SendWallops(x)         ((x)->flags & FLAGS_WALLOP)
#define SendServNotice(x)      ((x)->flags & FLAGS_SERVNOTICE)
#define NoPrivates(x)          ((x)->flags & FLAGS_NOPRIVATES)
#define NoCTCPDCC(x)           ((x)->flags & FLAGS_NOCTCPDCC)
#define IsRegged(x)            ((x)->flags & FLAGS_REGGED)
#define IsHideHost(x)          ((x)->flags & FLAGS_HIDEHOST)
#define IsGod(x)               ((x)->flags & FLAGS_GODMODE)
#define IsBot(x)               ((x)->flags & FLAGS_BOT)
#define IsSBot(x)              ((x)->flags & FLAGS_SBOT)
#define IsRCentral(x)          ((x)->flags & FLAGS_RCENTRAL)
#define IsAdmin(x)             ((x)->flags & FLAGS_ADMIN)
#define IsCoAdmin(x)           ((x)->flags & FLAGS_COADMIN)
#define IsBackupOp(x)          ((x)->flags & FLAGS_BACKUPOP)
#define IsCoder(x)             ((x)->flags & FLAGS_CODER)
#define IsHelper(x)            ((x)->flags & FLAGS_HELPER)
#define IsUserCom(x)           ((x)->flags & FLAGS_USERCOM)
#define IsWebCom(x)            ((x)->flags & FLAGS_WEBCOM)
#define IsBService(x)          ((x)->flags & FLAGS_BSERVICE)
#define IsCService(x)          ((x)->flags & FLAGS_CSERVICE)
#define IsEService(x)          ((x)->flags & FLAGS_ESERVICE)
#define IsWhoisNotice(x)       ((x)->flags & FLAGS_WHOIS)

#define IsDead(x)              ((x)->otherflags & FLAGS_DEADSOCKET)
#define IsUnixSocket(x)        ((x)->otherflags & FLAGS_UNIX)
#define IsListening(x)         ((x)->otherflags & FLAGS_LISTEN)
#define DoAccess(x)            ((x)->otherflags & FLAGS_CHKACCESS)
#define DoingDNS(x)            ((x)->otherflags & FLAGS_DOINGDNS)
#define DoingAuth(x)           ((x)->otherflags & FLAGS_AUTH)
#define IsLocal(x)             ((x)->otherflags & FLAGS_LOCAL)
#define NoNewLine(x)           ((x)->otherflags & FLAGS_NONL)
#define DoPing(x)              ((x)->otherflags & FLAGS_PING)
#define AskedPing(x)           ((x)->otherflags & FLAGS_ASKEDPING)
#define IsJunction(x)          ((x)->otherflags & FLAGS_JUNCTION)
#define IsBurst(x)             ((x)->otherflags & FLAGS_BURST)
#define IsBurstAck(x)          ((x)->otherflags & FLAGS_BURST_ACK)
#define IsBurstOrBurstAck(x)   ((x)->otherflags & (FLAGS_BURST|FLAGS_BURST_ACK))
#define IsIPChecked(x)         ((x)->otherflags & FLAGS_IPCHECK)
#define DoingSocks(x)          ((x)->otherflags & FLAGS_SOCKS)
#define DoingWrSocks(x)        ((x)->otherflags & FLAGS_WRSOCKS)
#define IsVlined(x)            ((x)->otherflags & FLAGS_VLINED)


#define SetDebug(x)            ((x)->flags |= FLAGS_DEBUG)
#define SetOper(x)             ((x)->flags |= FLAGS_OPER)
#define SetLocOp(x)            ((x)->flags |= FLAGS_LOCOP)
#define SetInvisible(x)        ((x)->flags |= FLAGS_INVISIBLE)
#define SetWallops(x)          ((x)->flags |= FLAGS_WALLOP)
#define SetNoPriv(x)           ((x)->flags |= FLAGS_NOPRIVATES)
#define SetNoCTCP(x)           ((x)->flags |= FLAGS_NOCTCPDCC)
#define SetRegged(x)           ((x)->flags |= FLAGS_REGGED)
#define SetHideHost(x)         ((x)->flags |= FLAGS_HIDEHOST)
#define SetGod(x)              ((x)->flags |= FLAGS_GODMODE)
#define SetBot(x)              ((x)->flags |= FLAGS_BOT)
#define SetSBot(x)             ((x)->flags |= FLAGS_SBOT)
#define SetRCentral(x)         ((x)->flags |= FLAGS_RCENTRAL)
#define SetAdmin(x)            ((x)->flags |= FLAGS_ADMIN)
#define SetCoAdmin(x)          ((x)->flags |= FLAGS_COADMIN)
#define SetBackupOp(x)         ((x)->flags |= FLAGS_BACKUPOP)
#define SetCoder(x)            ((x)->flags |= FLAGS_CODER)
#define SetHelper(x)           ((x)->flags |= FLAGS_HELPER)
#define SetUserCom(x)          ((x)->flags |= FLAGS_USERCOM)
#define SetWebCom(x)           ((x)->flags |= FLAGS_WEBCOM)
#define SetBService(x)         ((x)->flags |= FLAGS_BSERVICE)
#define SetCService(x)         ((x)->flags |= FLAGS_CSERVICE)
#define SetEService(x)         ((x)->flags |= FLAGS_ESERVICE)

#define SetUnixSock(x)         ((x)->otherflags |= FLAGS_UNIX)
#define SetAccess(x)           ((x)->otherflags |= FLAGS_CHKACCESS)
#define SetDNS(x)              ((x)->otherflags |= FLAGS_DOINGDNS)
#define SetAuth(x)             ((x)->otherflags |= FLAGS_AUTH)
#define SetAskedPing(x)        ((x)->otherflags |= FLAGS_ASKEDPING)
#define SetJunction(x)         ((x)->otherflags |= FLAGS_JUNCTION)
#define SetBurst(x)            ((x)->otherflags |= FLAGS_BURST)
#define SetBurstAck(x)         ((x)->otherflags |= FLAGS_BURST_ACK)
#define SetIPChecked(x)        ((x)->otherflags |= FLAGS_IPCHECK)
#define SetSocks(x)            ((x)->otherflags |= FLAGS_SOCKS)
#define SetWrSocks(x)          ((x)->otherflags |= FLAGS_WRSOCKS)
#define SetVlined(x)           ((x)->otherflags |= FLAGS_VLINED)


#define ClearDebug(x)          ((x)->flags &= ~FLAGS_DEBUG)
#define ClearOper(x)           ((x)->flags &= ~FLAGS_OPER)
#define ClearLocOp(x)          ((x)->flags &= ~FLAGS_LOCOP)
#define ClearInvisible(x)      ((x)->flags &= ~FLAGS_INVISIBLE)
#define ClearWallops(x)        ((x)->flags &= ~FLAGS_WALLOP)
#define ClearNoPriv(x)         ((x)->flags &= ~FLAGS_NOPRIVATES)
#define ClearNoCTCP(x)         ((x)->flags &= ~FLAGS_NOCTCPDCC)
#define ClearRegged(x)         ((x)->flags &= ~FLAGS_REGGED)
#define ClearHideHost(x)       ((x)->flags &= ~FLAGS_HIDEHOST)
#define ClearGod(x)            ((x)->flags &= ~FLAGS_GODMODE)
#define ClearBot(x)            ((x)->flags &= ~FLAGS_BOT)
#define ClearSBot(x)           ((x)->flags &= ~FLAGS_SBOT)
#define ClearRCentral(x)       ((x)->flags &= ~FLAGS_RCENTRAL)
#define ClearAdmin(x)          ((x)->flags &= ~FLAGS_ADMIN)
#define ClearCoAdmin(x)        ((x)->flags &= ~FLAGS_COADMIN)
#define ClearBackupOp(x)       ((x)->flags &= ~FLAGS_BACKUPOP)
#define ClearCoder(x)          ((x)->flags &= ~FLAGS_CODER)
#define ClearHelper(x)         ((x)->flags &= ~FLAGS_HELPER)
#define ClearUserCom(x)        ((x)->flags &= ~FLAGS_USERCOM)
#define ClearWebCom(x)         ((x)->flags &= ~FLAGS_WEBCOM)
#define ClearBService(x)       ((x)->flags &= ~FLAGS_BSERVICE)
#define ClearCService(x)       ((x)->flags &= ~FLAGS_CSERVICE)
#define ClearEService(x)       ((x)->flags &= ~FLAGS_ESERVICE)

#define ClearAccess(x)         ((x)->otherflags &= ~FLAGS_CHKACCESS)
#define ClearDNS(x)            ((x)->otherflags &= ~FLAGS_DOINGDNS)
#define ClearAuth(x)           ((x)->otherflags &= ~FLAGS_AUTH)
#define ClearPing(x)           ((x)->otherflags &= ~FLAGS_PING)
#define ClearAskedPing(x)      ((x)->otherflags &= ~FLAGS_ASKEDPING)
#define ClearBurst(x)          ((x)->otherflags &= ~FLAGS_BURST)
#define ClearBurstAck(x)       ((x)->otherflags &= ~FLAGS_BURST_ACK)
#define ClearSocks(x)          ((x)->otherflags &= ~FLAGS_SOCKS)
#define ClearWrSocks(x)        ((x)->otherflags &= ~FLAGS_WRSOCKS)
#define ClearVlined(x)         ((x)->otherflags &= ~FLAGS_VLINED)


/* used for async dns values */

#define ASYNC_NONE	0
#define ASYNC_CLIENT	1
#define ASYNC_CONNECT	2
#define ASYNC_CONF	3
#define ASYNC_PING	4

/* server notice stuff */

#define SNO_ADD		1
#define SNO_DEL		2
#define SNO_SET		3
				/* DON'T CHANGE THESE VALUES ! */
				/* THE CLIENTS DEPEND ON IT  ! */
#define SNO_OLDSNO	0x1	/* unsorted old messages */
#define SNO_SERVKILL	0x2	/* server kills (nick collisions) */
#define SNO_OPERKILL	0x4	/* oper kills */
#define SNO_HACK2	0x8	/* desyncs */
#define SNO_HACK3	0x10	/* temporary desyncs */
#define SNO_UNAUTH	0x20	/* unauthorized connections */
#define SNO_TCPCOMMON	0x40	/* common TCP or socket errors */
#define SNO_TOOMANY	0x80	/* too many connections */
#define SNO_HACK4	0x100	/* Uworld actions on channels */
#define SNO_GLINE	0x200	/* glines */
#define SNO_NETWORK	0x400	/* net join/break, etc */
#define SNO_IPMISMATCH	0x800	/* IP mismatches */
#define SNO_THROTTLE	0x1000	/* host throttle add/remove notices */
#define SNO_OLDREALOP	0x2000	/* old oper-only messages */
#define SNO_CONNEXIT	0x4000	/* client connect/exit (ugh) */
#define SNO_LOCGLOBOPS	0x8000	/* LOCOPS / CHATOPS / GLOBOPS */

#define SNO_ALL		0xffff	/* Don't make it larger then significant,
				 * that looks nicer */

#define SNO_USER	(SNO_ALL & ~SNO_OPER)

#define SNO_DEFAULT (SNO_NETWORK|SNO_OPERKILL|SNO_GLINE)
#define SNO_OPERDEFAULT (SNO_DEFAULT|SNO_HACK2|SNO_HACK4|SNO_THROTTLE|SNO_OLDSNO|SNO_CONNEXIT|SNO_LOCGLOBOPS)
#define SNO_OPER (SNO_CONNEXIT|SNO_OLDREALOP|SNO_LOCGLOBOPS)
#define SNO_NOISY (SNO_SERVKILL|SNO_UNAUTH)

/*
 * simple defines to differentiate between a tty and socket for
 * add_connection()  -Simon
 */

#define ADCON_TTY 0
#define ADCON_SOCKET 1

/*=============================================================================
 * Proto types
 */

extern int setsnomask(aClient *cptr, snomask_t newmask, int what);
extern snomask_t umode_make_snomask(snomask_t oldmask, char *arg, int what);
extern int connect_server(aConfItem *aconf, aClient *by, struct hostent *hp);
extern void report_error(char *text, aClient *cptr);
extern int inetport(aClient *cptr, char *name, unsigned short int port);
extern int add_listener(aConfItem *aconf);
extern void close_listeners(void);
extern void init_sys(void);
extern void write_pidfile(void);
extern enum AuthorizationCheckResult check_client(aClient *cptr);
extern int check_server(aClient *cptr);
extern void close_connection(aClient *cptr);
extern int get_sockerr(aClient *cptr);
extern void set_non_blocking(int fd, aClient *cptr);
extern aClient *add_connection(aClient *cptr, int fd, int type);
extern int read_message(time_t delay);
extern void get_my_name(aClient *cptr, char *name, size_t len);
extern int setup_ping(void);

extern int highest_fd, resfd;
extern unsigned int readcalls;
extern aClient *loc_clients[MAXCONNECTIONS];
#ifdef VIRTUAL_HOST
extern struct sockaddr_in vserv;
#endif

#endif /* S_BSD_H */
