/*
 * RaptorIRCD - Internet Relay Chat Daemon, ircd/whowas.c
 * Copyright (C) 2001 Markku Savela
 *
 * See file doc/authors for additional names of
 * the programmers.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "sys.h"
#include <stdlib.h>
#include "common.h"
#include "h.h"
#include "struct.h"
#include "numeric.h"
#include "send.h"
#include "s_misc.h"
#include "s_err.h"
#include "ircd.h"
#include "s_user.h"
#include "s_serv.h"
#include "hash.h"
#include "fileio.h"
#include "s_conf.h"
#include "raptor.h"

RCSTAG_CC("$Id: raptor.c,v 1.1.1.1 2001/05/06 13:06:02 tterm Exp $");

#ifdef USE_SC_CS
/*
 * The m_?? code was based on Unreal3.1+sf
*/
int m_cs(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
  aClient *acptr;
  char service[NICKLEN + 1];
  char msg[BUFSIZE];
  int par;

  strcpy(service, ChanServ);

  if (parc < 2 || *parv[1] == '\0')
  {
    sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
    return -1;
  }

  if (*parv[1] == ':')
  {
    sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
    return -1;
  }

  strcpy(msg, parv[1]);
  for (par = 2; par < parc; par++)
  {
    strcpy(&msg[strlen(msg)], " ");
    strcpy(&msg[strlen(msg)], parv[par]);
  }

  if ((acptr = FindClient(service)))
    if (!MyConnect(acptr))
      sendto_one(acptr, ":%s PRIVMSG %s@%s :%s", parv[0],
          service, ChanServ_SERVER, msg);
    else
      sendto_one(sptr, err_str(ERR_SERVICESDOWN), me.name, parv[0], service);
  else
    sendto_one(sptr, err_str(ERR_SERVICESDOWN), me.name, parv[0], service);

  return 0;
}
#endif


#ifdef USE_SC_HS
int m_hs(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
  aClient *acptr;
  char service[NICKLEN + 1];
  char msg[BUFSIZE];
  int par;

  strcpy(service, HelpServ);

  if (parc < 2 || *parv[1] == '\0')
  {
    sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
    return -1;
  }

  if (*parv[1] == ':')
  {
    sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
    return -1;
  }

  strcpy(msg, parv[1]);
  for (par = 2; par < parc; par++)
  {
    strcpy(&msg[strlen(msg)], " ");
    strcpy(&msg[strlen(msg)], parv[par]);
  }

  if ((acptr = FindClient(service)))
    if (!MyConnect(acptr))
      sendto_one(acptr, ":%s PRIVMSG %s@%s :%s", parv[0],
          service, HelpServ_SERVER, msg);
    else
      sendto_one(sptr, err_str(ERR_SERVICESDOWN), me.name, parv[0], service);
  else
    sendto_one(sptr, err_str(ERR_SERVICESDOWN), me.name, parv[0], service);

  return 0;
}
#endif


#ifdef USE_SC_MS
int m_ms(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
  aClient *acptr;
  char service[NICKLEN + 1];
  char msg[BUFSIZE];
  int par;

  strcpy(service, MemoServ);

  if (parc < 2 || *parv[1] == '\0')
  {
    sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
    return -1;
  }

  if (*parv[1] == ':')
  {
    sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
    return -1;
  }

  strcpy(msg, parv[1]);
  for (par = 2; par < parc; par++)
  {
    strcpy(&msg[strlen(msg)], " ");
    strcpy(&msg[strlen(msg)], parv[par]);
  }

  if ((acptr = FindClient(service)))
    if (!MyConnect(acptr))
      sendto_one(acptr, ":%s PRIVMSG %s@%s :%s", parv[0],
          service, MemoServ_SERVER, msg);
    else
      sendto_one(sptr, err_str(ERR_SERVICESDOWN), me.name, parv[0], service);
  else
    sendto_one(sptr, err_str(ERR_SERVICESDOWN), me.name, parv[0], service);

  return 0;
}
#endif


#ifdef USE_SC_NS
int m_ns(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
  aClient *acptr;
  char service[NICKLEN + 1];
  char msg[BUFSIZE];
  int par;

  strcpy(service, NickServ);

  if (parc < 2 || *parv[1] == '\0')
  {
    sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
    return -1;
  }

  if (*parv[1] == ':')
  {
    sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
    return -1;
  }

  strcpy(msg, parv[1]);
  for (par = 2; par < parc; par++)
  {
    strcpy(&msg[strlen(msg)], " ");
    strcpy(&msg[strlen(msg)], parv[par]);
  }

  if ((acptr = FindClient(service)))
    if (!MyConnect(acptr))
      sendto_one(acptr, ":%s PRIVMSG %s@%s :%s", parv[0],
          service, NickServ_SERVER, msg);
    else
      sendto_one(sptr, err_str(ERR_SERVICESDOWN), me.name, parv[0], service);
  else
    sendto_one(sptr, err_str(ERR_SERVICESDOWN), me.name, parv[0], service);

  return 0;
}
#endif


#ifdef USE_SC_OS
int m_os(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
  aClient *acptr;
  char service[NICKLEN + 1];
  char msg[BUFSIZE];
  int par;

  strcpy(service, OperServ);

  if (parc < 2 || *parv[1] == '\0')
  {
    sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
    return -1;
  }

  if (*parv[1] == ':')
  {
    sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
    return -1;
  }

  strcpy(msg, parv[1]);
  for (par = 2; par < parc; par++)
  {
    strcpy(&msg[strlen(msg)], " ");
    strcpy(&msg[strlen(msg)], parv[par]);
  }

  if ((acptr = FindClient(service)))
    if (!MyConnect(acptr))
      sendto_one(acptr, ":%s PRIVMSG %s@%s :%s", parv[0],
          service, OperServ_SERVER, msg);
    else
      sendto_one(sptr, err_str(ERR_SERVICESDOWN), me.name, parv[0], service);
  else
    sendto_one(sptr, err_str(ERR_SERVICESDOWN), me.name, parv[0], service);

  return 0;
}
#endif


#ifdef USE_SC_SS
int m_ss(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
  aClient *acptr;
  char service[NICKLEN + 1];
  char msg[BUFSIZE];
  int par;

  strcpy(service, StatServ);

  if (parc < 2 || *parv[1] == '\0')
  {
    sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
    return -1;
  }

  if (*parv[1] == ':')
  {
    sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
    return -1;
  }

  strcpy(msg, parv[1]);
  for (par = 2; par < parc; par++)
  {
    strcpy(&msg[strlen(msg)], " ");
    strcpy(&msg[strlen(msg)], parv[par]);
  }

  if ((acptr = FindClient(service)))
    if (!MyConnect(acptr))
      sendto_one(acptr, ":%s PRIVMSG %s@%s :%s", parv[0],
          service, StatServ_SERVER, msg);
    else
      sendto_one(sptr, err_str(ERR_SERVICESDOWN), me.name, parv[0], service);
  else
    sendto_one(sptr, err_str(ERR_SERVICESDOWN), me.name, parv[0], service);

  return 0;
}
#endif


#ifdef USE_SC_X2
int m_x2(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
  aClient *acptr;
  char service[NICKLEN + 1];
  char msg[BUFSIZE];
  int par;

  strcpy(service, X2);

  if (parc < 2 || *parv[1] == '\0')
  {
    sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
    return -1;
  }

  if (*parv[1] == ':')
  {
    sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
    return -1;
  }

  strcpy(msg, parv[1]);
  for (par = 2; par < parc; par++)
  {
    strcpy(&msg[strlen(msg)], " ");
    strcpy(&msg[strlen(msg)], parv[par]);
  }

  if ((acptr = FindClient(service)))
    if (!MyConnect(acptr))
      sendto_one(acptr, ":%s PRIVMSG %s@%s :%s", parv[0],
          service, X2_SERVER, msg);
    else
      sendto_one(sptr, err_str(ERR_SERVICESDOWN), me.name, parv[0], service);
  else
    sendto_one(sptr, err_str(ERR_SERVICESDOWN), me.name, parv[0], service);

  return 0;
}
#endif


#ifdef USE_SC_LOCAL_LINES
int m_addline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{

  return 0;
}


int m_remline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{

  return 0;
}


int m_viewline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{

  return 0;
}
#endif /* USE_SC_LOCAL_LINES */


#ifdef USE_SC_REMOTE_LINES
int m_addhline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{

  return 0;
}


int m_addqline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{

  return 0;
}


int m_adduline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{

  return 0;
}


int m_disableo(aClient *cptr, aClient *sptr, int parc, char *parv[])
{

  return 0;
}


int m_enableo(aClient *cptr, aClient *sptr, int parc, char *parv[])
{

  return 0;
}


int m_remhline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{

  return 0;
}


int m_remqline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{

  return 0;
}


int m_remuline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{

  return 0;
}


int m_grehash(aClient *cptr, aClient *sptr, int parc, char *parv[])
{

  return 0;
}


int m_sjoin(aClient *cptr, aClient *sptr, int parc, char *parv[])
{

  return 0;
}


int m_spart(aClient *cptr, aClient *sptr, int parc, char *parv[])
{

  return 0;
}
#endif /* USE_SC_REMOTE_LINES */


#ifdef USE_RULES
/*
 * m_rules
 *
 * parv[0] - sender prefix
 * parv[1] - servername
 *
 * most of this code was reused from the m_motd command
 * [Term] - 2001/02/21th
 */
int m_rules(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
  struct tm *tm = &motd_tm;	/* Default: Most general case */
  atrecord *ptr;
  int count;
  register aMotdItem *temp;

  if (hunt_server(0, cptr, sptr, ":%s RULES :%s", 1, parc,
	parv) != HUNTED_ISME)
    return 0;

  /*
   * Find out if this is a remote query or if we have a T line for our hostname
   */
  if (IsServer(cptr))
  {
    tm = NULL;			/* Remote RULES */
    temp = rules;
  }
  else
  {
    for (ptr = tdata; ptr; ptr = ptr->next)
    {
      if (!match(ptr->hostmask, cptr->sockhost))
	break;
    }
    if (ptr)
    {
      temp = ptr->tmotd;
      tm = &ptr->tmotd_tm;
    }
    else
      temp = rules;
  }
  if (temp == NULL)
  {
    sendto_one(sptr, err_str(ERR_NORULES), me.name, parv[0]);
    return 0;
  }
  if (tm)			/* Not remote? */
  {
    sendto_one(sptr, rpl_str(RPL_RULESSTART), me.name, parv[0], me.name);
    count = 100;
  }
  else
    count = 100;
  for (; temp; temp = temp->next)
  {
    sendto_one(sptr, rpl_str(RPL_RULES), me.name, parv[0], temp->line);
    if (--count == 0)
	break;
  }
  sendto_one(sptr, rpl_str(RPL_ENDOFRULES), me.name, parv[0]);
  return 0;
}

#endif /* USE_RULES */