/*
 * RaptorIRCD - Internet Relay Chat Daemon, ircd/s_socks.c
 * Copyright (C) 1992 Darren Reed
 *
 * See file doc/authors for additional names of
 * the programmers.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "sys.h"
#include <sys/socket.h>
#if HAVE_SYS_FILE_H
#include <sys/file.h>
#endif
#if HAVE_SYS_IOCTL_H
#include <sys/ioctl.h>
#endif
#ifdef	UNIXPORT
#include <sys/un.h>
#endif
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HPUX
#include <arpa/inet.h>
#endif /* HPUX */
#if HAVE_FCNTL_H
#include <fcntl.h>
#endif
#ifdef USE_SYSLOG
#include <syslog.h>
#endif
#include <pwd.h>
#include <sys/types.h>

#include "h.h"
#include "res.h"
#include "struct.h"
#include "common.h"
#include "send.h"
#include "s_bsd.h"
#include "s_misc.h"
#include "support.h"
#include "ircd.h"
#include "s_socks.h"
#include "sprintf_irc.h"
#include "s_conf.h"
#include "IPcheck.h"

RCSTAG_CC("$Id: s_socks.c,v 1.1.1.1 2001/05/06 13:06:02 tterm Exp $");

/* Not very nice way of dealing with this - but what the heck :> */

#ifdef NEW_CONNECTSYS
/*
 * start_socks (new system)
 *
 * Flag the client to show that an attempt to contact the socks server on
 * the client's host. The connect and subsequently the socket are all put
 * into 'non-blocking' mode.
 */
void start_socks(aClient *cptr)
{
  struct sockaddr_in sock;
  struct timeval wait;
  int err;
  int rc;
  char text[1024];

  if (IPcheck_proxy_cache_get(cptr))
    {
#ifdef CONNECT_MSGS
      sprintf_irc(text, ":%s NOTICE SOCKS :*** Proxy test passed (cached)\r\n",  me.name);
      write(cptr->fd, text, strlen(text));
#endif
      if (!DoingDNS(cptr) && !DoingAuth(cptr))
	SetAccess(cptr);
      return;
    }

  if (find_socksexception(cptr->sockhost)) {
#ifdef CONNECT_MSGS
    sprintf_irc(text, ":%s NOTICE SOCKS :*** Proxy test passed (exception)\r\n", me.name ); 
    write(cptr->fd, text, strlen(text));
#endif
    return;
  }

  Debug((DEBUG_NOTICE, "start_socks(%p) fd %d status %d",
      cptr, cptr->fd, cptr->status));

  alarm(2);
  cptr->socksfd = socket(AF_INET, SOCK_STREAM, 0);
  err = errno;
  alarm(0);
  if (cptr->socksfd < 0 && err == EAGAIN)
    sendto_ops("Can't allocate fd for socks on %s : socket: No more sockets",
	get_client_name(cptr, TRUE));

  if (cptr->socksfd < 0)
  {
#ifdef	USE_SYSLOG
    syslog(LOG_ERR, "Unable to create socks socket for %s:%m",
	get_client_name(cptr, TRUE));
#endif
    Debug((DEBUG_ERROR, "Unable to create socks socket for %s:%s",
	get_client_name(cptr, TRUE), strerror(get_sockerr(cptr))));
    if (!DoingDNS(cptr) && !DoingAuth(cptr))
      SetAccess(cptr);
    ircstp->is_abad++;
    return;
  }
  if (cptr->socksfd >= (MAXCONNECTIONS - 2))
  {
    sendto_ops("Can't allocate fd for socks on %s", get_client_name(cptr, TRUE));
    close(cptr->socksfd);
    return;
  }

  set_non_blocking(cptr->socksfd, cptr);

  memcpy(&sock.sin_addr, &cptr->ip, sizeof(struct in_addr));

  sock.sin_port = htons(1080);
  sock.sin_family = AF_INET;
#ifdef CONNECT_MSGS
  sprintf_irc(text, ":%s NOTICE SOCKS :*** Checking WinGate/SocksProxy security...\r\n", me.name ); 
  write(cptr->fd, text, strlen(text));
#endif

  /* Rewritten because of long delays with firewalls, etc. - [Kirk]     */
  /* Needs testing on other OS's, unsure of performances on OS's other  */
  /* than FreeBSD and Linux                                             */

  /* Let's try the connect */
  rc = connect(cptr->socksfd, (struct sockaddr *)&sock, sizeof(sock));
  if (rc >= 0) {
    /* Connected successfully straight away */
    SetSocks(cptr);
    SetWrSocks(cptr);
    if (cptr->socksfd > highest_fd)
      highest_fd = cptr->socksfd;
    return;
  } else if (errno == EINPROGRESS) {
    /* Connection is in progress, it's taking it's time, let's be nasty */
    /* and set a timeout because it's a slow barsteward - below */
  } else {
    /* Couldn't connect */
    ircstp->is_abad++;
    close(cptr->socksfd);
    cptr->socksfd = -1;
#ifdef CONNECT_MSGS
    sprintf_irc(text, ":%s NOTICE SOCKS :*** Proxy test passed\r\n", me.name ); 
    write(cptr->fd, text, strlen(text));
#endif

    IPcheck_proxy_cache_set(cptr, !0);

    if (!DoingDNS(cptr) && !DoingAuth(cptr))
      SetAccess(cptr);
    return;
  }

  {
    fd_set fds;
    FD_ZERO(&fds);
    FD_SET(cptr->socksfd, &fds);
    wait.tv_sec = 5;            /* A timeout of 5 seconds should be fine. */
    wait.tv_usec = 0;
    rc = select(cptr->socksfd+1, NULL, &fds, NULL, &wait);
  }

  if (rc > 0) {
    /* select() has returned that descriptor is ready */
  } else if (rc == 0) {
    /* select() has returned 0, we'll pass the socks scan */
    ircstp->is_abad++;
    close(cptr->socksfd);
    cptr->socksfd = -1;
#ifdef CONNECT_MSGS
    sprintf_irc(text, ":%s NOTICE SOCKS :*** Proxy test passed\r\n", me.name ); 
    write(cptr->fd, text, strlen(text));
#endif

    IPcheck_proxy_cache_set(cptr, !0);

    if (!DoingDNS(cptr) && !DoingAuth(cptr))
      SetAccess(cptr);
    return;
  } else {
    /* select() has buggered up, we'll pass the socks scan */
    ircstp->is_abad++;
    close(cptr->socksfd);
    cptr->socksfd = -1;
#ifdef CONNECT_MSGS
    sprintf_irc(text, ":%s NOTICE SOCKS :*** Proxy test passed\r\n", me.name ); 
    write(cptr->fd, text, strlen(text));
#endif

    IPcheck_proxy_cache_set(cptr, !0);

    if (!DoingDNS(cptr) && !DoingAuth(cptr))
      SetAccess(cptr);
    return;
  }

  /* Let's see if we connected or not */
  rc = 0;
  {
    int arglen = sizeof(int);
    if (getsockopt(cptr->socksfd, SOL_SOCKET, SO_ERROR, &rc, &arglen) < 0)
      rc = errno;
  }

  if (rc) {
    ircstp->is_abad++;
    close(cptr->socksfd);
    cptr->socksfd = -1;
#ifdef CONNECT_MSGS
    sprintf_irc(text, ":%s NOTICE SOCKS :*** Proxy test passed\r\n", me.name ); 
    write(cptr->fd, text, strlen(text));
#endif

    IPcheck_proxy_cache_set(cptr, !0);

    if (!DoingDNS(cptr) && !DoingAuth(cptr))
      SetAccess(cptr);
    return;
  } else {
    SetSocks(cptr);
    SetWrSocks(cptr);
    if (cptr->socksfd > highest_fd)
      highest_fd = cptr->socksfd;
    return;
  }
}

#else /* NEW_CONNECTSYS */

/*
 * start_socks
 *
 * Flag the client to show that an attempt to contact the socks server on
 * the client's host.  The connect and subsequently the socket are all put
 * into 'non-blocking' mode.
 */
void start_socks(aClient *cptr)
{
  struct sockaddr_in sock;
  int err;
  char text[1024];

  if (IPcheck_proxy_cache_get(cptr))
    {
#ifdef CONNECT_MSGS
      sprintf_irc(text, ":%s NOTICE SOCKS :*** Proxy test passed (cached)\r\n", me.name);
      write(cptr->fd, text, strlen(text));
#endif
      if (!DoingDNS(cptr) && !DoingAuth(cptr))
        SetAccess(cptr);
      return;
    }

  if (find_socksexception(cptr->sockhost)) {
#ifdef CONNECT_MSGS
    sprintf_irc(text, ":%s NOTICE SOCKS :*** Proxy test passed (exception)\r\n", me.name);
    write(cptr->fd, text, strlen(text));
#endif
    return;
  }

  Debug((DEBUG_NOTICE, "start_socks(%p) fd %d status %d",
      cptr, cptr->fd, cptr->status));

  alarm(2);
  cptr->socksfd = socket(AF_INET, SOCK_STREAM, 0);
  err = errno;
  alarm(0);
  if (cptr->socksfd < 0 && err == EAGAIN)
    sendto_ops("Can't allocate fd for socks on %s : socket: No more sockets",
        get_client_name(cptr, TRUE));

  if (cptr->socksfd < 0)
  {
#ifdef  USE_SYSLOG
    syslog(LOG_ERR, "Unable to create socks socket for %s:%m",
        get_client_name(cptr, TRUE));
#endif
    Debug((DEBUG_ERROR, "Unable to create socks socket for %s:%s",
        get_client_name(cptr, TRUE), strerror(get_sockerr(cptr))));
    if (!DoingDNS(cptr) && !DoingAuth(cptr))
      SetAccess(cptr);
    ircstp->is_abad++;
    return;
  }
  if (cptr->socksfd >= (MAXCONNECTIONS - 2))
  {
    sendto_ops("Can't allocate fd for socks on %s", get_client_name(cptr, TRUE));
    close(cptr->socksfd);
    return;
  }

  set_non_blocking(cptr->socksfd, cptr);

  memcpy(&sock.sin_addr, &cptr->ip, sizeof(struct in_addr));

  sock.sin_port = htons(1080);
  sock.sin_family = AF_INET;
#ifdef CONNECT_MSGS
  sprintf_irc(text, ":%s NOTICE SOCKS :*** Checking WinGate/SocksProxy security...\r\n", me.name);
  write(cptr->fd, text, strlen(text));
#endif
  alarm((unsigned)4);
  if (connect(cptr->socksfd, (struct sockaddr *)&sock,
      sizeof(sock)) == -1 && errno != EINPROGRESS)
  {
    ircstp->is_abad++;
    /*
     * No error report from this...
     */
    alarm((unsigned)0);
    close(cptr->socksfd);
    cptr->socksfd = -1;
#ifdef CONNECT_MSGS
    sprintf_irc(text, ":%s NOTICE SOCKS :*** Proxy test passed\r\n", me.name );
    write(cptr->fd, text, strlen(text));
#endif

    IPcheck_proxy_cache_set(cptr, !0);

    if (!DoingDNS(cptr) && !DoingAuth(cptr))
      SetAccess(cptr);
    return;
  }
  alarm((unsigned)0);
  SetSocks(cptr);
  SetWrSocks(cptr);
  if (cptr->socksfd > highest_fd)
    highest_fd = cptr->socksfd;
  return;
}
#endif /* NEW_CONNECTSYS */


/*
 * send_socksports
 *
 */
void send_socksports(aClient *cptr)
{
  struct sockaddr_in us, them;
  char socksbuf[255], *uname;
  size_t ulen, tlen;
  struct passwd *pw;
#ifdef SOCKS5
  struct socks5out *sk;
#else
  struct socks4 *sk;
#endif
  char text[1024];

  Debug((DEBUG_NOTICE, "write_socksports(%p) fd %d socksfd %d stat %d",
      cptr, cptr->fd, cptr->socksfd, cptr->status));
  tlen = ulen = sizeof(us);
  if (getsockname(cptr->fd, (struct sockaddr *)&us, &ulen) ||
      getpeername(cptr->fd, (struct sockaddr *)&them, &tlen))
  {
#ifdef	USE_SYSLOG
    syslog(LOG_ERR, "socks get{sock,peer}name error for %s:%m",
	get_client_name(cptr, TRUE));
#endif
    goto sockssenderr;
  }

  if( ! (pw=getpwuid(getuid())) ) {
#ifdef	USE_SYSLOG
    syslog(LOG_ERR, "socks getpwuid error");
#endif
    goto sockssenderr;
  }

#ifdef SOCKS5
  sk=(struct socks5out *)socksbuf;
  uname=socksbuf+sizeof(struct socks5out);

  sk->ver = 5;
  sk->nmethods = 1;
  sk->methods = 0;
  strcpy(uname, pw->pw_name);

  Debug((DEBUG_SEND, "sending connection request to socks port %s.1080",
         inetntoa(them.sin_addr)));
  if (write(cptr->socksfd, socksbuf, sizeof(struct socks5out)+strlen(uname)+1)
      != (int)(sizeof(struct socks5out)+strlen(uname)+1) )
  {
#else /* SOCKS5 */
  sk=(struct socks4 *)socksbuf;
  uname=socksbuf+sizeof(struct socks4);

  sk->vn=4;
  sk->cd=1;
  sk->dstport=us.sin_port;
  sk->dstip=us.sin_addr.s_addr;
  strcpy(uname, pw->pw_name);

  Debug((DEBUG_SEND, "sending connection request to socks port %s.1080",
         inetntoa(them.sin_addr)));
  if (write(cptr->socksfd, socksbuf, sizeof(struct socks4)+strlen(uname)+1)
      != (int)(sizeof(struct socks4)+strlen(uname)+1) )
  {
#endif /* SOCKS5 */
  sockssenderr:
#ifdef CONNECT_MSGS
    sprintf_irc(text, ":%s NOTICE SOCKS :*** Proxy test passed\r\n", me.name ); 
    write(cptr->fd, text, strlen(text));
#endif

    IPcheck_proxy_cache_set(cptr, !0);

    close(cptr->socksfd);
    if (cptr->socksfd == highest_fd)
      while (!loc_clients[highest_fd])
	highest_fd--;
    cptr->socksfd = -1;
    ClearSocks(cptr);
    if (!DoingDNS(cptr) && !DoingAuth(cptr))
      SetAccess(cptr);             
  }                                
  ClearWrSocks(cptr);
  return;
}

/*
 * read_socksports
 *
 * read the reply (if any) from the ident server we connected to.
 * The actual read processijng here is pretty weak - no handling of the reply
 * if it is fragmented by IP.
 */
void read_socksports(aClient *cptr)
{
  int len;
#ifdef SOCKS5
  struct socks5in *sk;
#else
  struct socks4 *sk;
#endif
  char socksbuf[255];
  char text[1024];

#ifdef SOCKS5
  sk=(struct socks5in *)socksbuf;

  Debug((DEBUG_NOTICE, "read_socksports(%p) fd %d socksfd %d stat %d",
      cptr, cptr->fd, cptr->socksfd, cptr->status));

  if ( cptr->socksfd>0 && read(cptr->socksfd, socksbuf, sizeof(struct socks5in)) == sizeof(struct socks5in) )
  {
      switch(sk->method)
      {
      case 0:
#else /* SOCKS5 */
  sk=(struct socks4 *)socksbuf;

  Debug((DEBUG_NOTICE, "read_socksports(%p) fd %d socksfd %d stat %d",
      cptr, cptr->fd, cptr->socksfd, cptr->status));

  if ( cptr->socksfd>0 && read(cptr->socksfd, socksbuf, sizeof(struct socks4)) == sizeof(struct socks4) )
  {
      switch(sk->cd)
      {
      case 90:
#endif /* SOCKS5 */
#ifdef CONNECT_MSGS
          sprintf_irc(text, ":%s NOTICE SOCKS :*** Proxy is UNSECURE\r\n", me.name ); 
          write(cptr->fd, text, strlen(text));
          sprintf_irc(text, ":%s NOTICE SOCKS :*** Please visit " URL_POLICY "\r\n", me.name ); 
          write(cptr->fd, text, strlen(text));
#endif
	  sendto_realops("UNSECURE SocksProxy or Wingate attempted connection [%s]", cptr->sockhost);

	  IPcheck_proxy_cache_set(cptr, 0);

         /* close connection */
          close(cptr->socksfd);
          close(cptr->authfd);
	  exit_client(cptr, cptr, &me, "Unsecure SocksProxy or WinGate - " URL_PROXY);
          return;
      default:
          break;
      }
  }

#ifdef CONNECT_MSGS
  sprintf_irc(text, ":%s NOTICE SOCKS :*** Proxy test passed\r\n", me.name );
  write(cptr->fd, text, strlen(text));
#endif

  IPcheck_proxy_cache_set(cptr, !0);

  cptr->lasttime = now;
  close(cptr->socksfd);
  if (cptr->socksfd == highest_fd)
    while (!loc_clients[highest_fd])
      highest_fd--;
  cptr->socksfd = -1;
  ClearSocks(cptr);
  ClearWrSocks(cptr);
  if (!DoingDNS(cptr) && !DoingAuth(cptr))
    SetAccess(cptr);
  return;
}

